/*
 ------------------------------------------------------------------------
 Thinstuff iRdesktop
 A RDP client for the iPhone and iPod Touch, based off WinAdmin
 (an iPhone RDP client by Carter Harrison) which is based off CoRD 
 (a Mac OS X RDP client by Craig Dooley and Dorian Johnson) which is in 
 turn based off of the Unix program rdesktop by Matthew Chapman.
 ------------------------------------------------------------------------
 
 PageView.h
 Copyright (C) Carter Harrison   2008-2009
 
 ------------------------------------------------------------------------
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ------------------------------------------------------------------------
 */


#import <UIKit/UIKit.h>
#import <QuartzCore/CoreAnimation.h>
#import "CALayerDelegate.h"

#define kPageControlDistanceFromBottom	80
#define kDistanceBetweenPagesPortrait	35.0
#define kPageTopPortrait		100.0
#define kPageWidthPortrait		160.0
#define kPageHeightPortrait		240.0
#define kTitleLabelTopPortrait			25.0

#define kDistanceBetweenPagesLandscape	40
#define kPageTopLandscape		70.0
#define kPageWidthLandscape		240.0
#define kPageHeightLandscape	160.0
#define kTitleLabelTopLandscape			10.0

#define HORIZ_SWIPE_DRAG_MIN  10
#define VERT_SWIPE_DRAG_MAX    25

@interface PageView : UIView 
{
	id delegate;
	BOOL isZoomed;
	UIPageControl *pageControl;
	UILabel *titleLabel;
	UILabel *subTitleLabel;
	NSMutableArray *views;
	NSMutableArray *closeButtons;
	CALayerDelegate *layerDelegate;
	
	CGPoint startTouchPosition;
	
	BOOL orientationChangeSinceLastLayout;
}

////////////////////
/// Initializers ///
////////////////////
- (id)initWithFrame:(CGRect)frame delegate:(id)object;

/////////////////
/// Accessors ///
/////////////////
- (int)currentPageNumber;
- (void)setOrientationChangeSinceLastLayout:(BOOL)value;

///////////////////////////////
/// Drawing Related Methods ///
///////////////////////////////
- (void)drawBackgroundGradient;
- (UIPageControl *)setupPageControl;
- (UILabel *)setupTitleLabel;
- (UILabel *)setupSubTitleLabel;
- (void)layoutPages;
- (void)scrollToPage:(int)pageNumber animated:(BOOL)animated andZoom:(BOOL)zoom;
- (void)removePageAtPageNumber:(int)pageNumber animated:(BOOL)animated;
- (void)userDidChangePage;
- (void)zoomCurrentPage;
- (void)unZoomCurrentPage;
- (BOOL)isZoomed;
- (void)hideCloseButtonForCurrentPageWithCallbackObject:(id)object selector:(SEL)selector;
- (void)showCloseButtonForCurrentPage;

////////////////////////////////
/// Delegate Related Methods ///
////////////////////////////////
- (void)setDelegate:(id)object;
- (id)delegate;

@end

@protocol PageViewDelegate
- (int)numberOfPagesToDisplay;
- (UIView *)viewForPageNumber:(int)pageNumber;
- (CGRect)rectForZoomedPage;
@optional
- (NSString *)titleForPageNumber:(int)page;
- (NSString *)subTitleForPageNumber:(int)page;
- (BOOL)shouldSelectPageNumber:(int)page;
- (void)willSelectPageNumber:(int)page;
- (void)didSelectPageNumber:(int)page;
- (void)willUnzoomCurrentPage;
- (void)didUnzoomCurrentPage;
- (void)willZoomCurrentPage;
- (void)didZoomCurrentPage;
- (void)userDidClosePageAtPageNumber:(int)pageNumber;
@end